
from django.test import TestCase
from django.core.management import call_command

from site_config.models import ConfigField
from site_config import SiteConfig


SITE_1_ID = 1
SITE_2_ID = 2

FIELD_TYPE_INPUT = 'input'
FIELD_TYPE_TEXT = 'text'
FIELD_TYPE_HTML = 'html'
FIELD_TYPE_JSON = 'json'

COMMA_SPLITTER = ','


class SiteConfigTestCase(TestCase):

    def setUp(self):
        call_command('sync_translation_fields', interactive=False)

    def test_config_returns_input_value(self):

        ConfigField.objects.create(
            site_id=SITE_1_ID,
            label='Test',
            name='test',
            type=FIELD_TYPE_INPUT,
            value_input='test value')

        config = SiteConfig()

        self.assertEqual(config.test, 'test value')

    def test_config_raises_error_when_field_is_from_other_site(self):

        ConfigField.objects.create(
            site_id=SITE_1_ID,
            label='Test',
            name='test',
            type=FIELD_TYPE_INPUT,
            value_input='test value')

        config = SiteConfig()

        self.assertRaises(AttributeError, lambda: config.test)

    def test_config_returns_correct_value_splitted_by_comma(self):

        ConfigField.objects.create(
            site_id=SITE_1_ID,
            label='Test',
            name='test',
            splitter=COMMA_SPLITTER,
            type=FIELD_TYPE_INPUT,
            value_input='test,value')

        config = SiteConfig()

        val = config.test

        self.assertEqual(len(val), 2)
        self.assertEqual(val[0], 'test')
        self.assertEqual(val[1], 'value')




from django.apps import apps, AppConfig
from django.utils.translation import ugettext_lazy as _
from django.utils.functional import cached_property
from django.conf import settings


default_app_config = 'site_config.SiteConfigApp'

__version__ = '3.3'

__all__ = ['default_app_config', 'SiteConfig']


class SiteConfigApp(AppConfig):
    name = 'site_config'
    verbose_name = _("Settings")


class SiteConfig(object):

    def __init__(self):
        self._updated_fields = []

    def __getattr__(self, name):
        if name.startswith('_'):
            return super(SiteConfig, self).__getattribute__(name)

        return self._get_field(name).value

    def __setattr__(self, key, value):
        if key.startswith('_'):
            super(SiteConfig, self).__setattr__(key, value)
        else:
            self._get_field(key).value = value
            self._updated_fields.append(key)

    @cached_property
    def _fields(self):
        field_model = apps.get_model('site_config', 'ConfigField')
        fields = field_model.objects.filter(site_id=settings.SITE_ID)
        return {f.name: f for f in fields}

    def _get_field(self, name):
        try:
            return self._fields[name]
        except KeyError:
            raise AttributeError("Site config has no field named '%s'" % name)

    def save(self):
        for f_name, field in self._fields.items():
            if f_name in self._updated_fields:
                field.save()

        self._updated_fields = []

    def reload(self):
        try:
            del self.__dict__['_fields']
        except KeyError:
            pass

config = SiteConfig()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-23 12:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('site_config', '0002_auto_20170714_1028'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='configfield',
            options={'ordering': ['label'], 'verbose_name': 'settings', 'verbose_name_plural': 'settings'},
        ),
        migrations.AddField(
            model_name='configfield',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='sites.Site', verbose_name='Site'),
        ),
        migrations.AlterField(
            model_name='configfield',
            name='group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='site_config.ConfigGroup', verbose_name='Group'),
        ),
        migrations.AlterField(
            model_name='configfield',
            name='type',
            field=models.CharField(choices=[(b'input', 'Input'), (b'text', 'Text'), (b'html', 'HTML'), (b'int', 'Integer'), (b'float', 'Float'), (b'bool', 'True / False'), (b'url', 'Url'), (b'email', 'Email'), (b'file', 'File'), (b'image', 'Image'), (b'json', 'JSON')], max_length=50, verbose_name='Type'),
        ),
        migrations.AlterField(
            model_name='configfield',
            name='value_input',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Input'),
        ),
        migrations.AlterUniqueTogether(
            name='configfield',
            unique_together=set([('site', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-07-14 10:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('site_config', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Config group',
                'verbose_name_plural': 'Config groups',
            },
        ),

        migrations.AddField(
            model_name='configfield',
            name='value_json',
            field=models.TextField(blank=True, max_length=10000, null=True, verbose_name='Text'),
        ),
        migrations.AlterField(
            model_name='configfield',
            name='splitter',
            field=models.CharField(blank=True, choices=[(b'newline', 'New line'), (b',', 'Comma'), (b'.', 'Dot'), (b';', 'Semicolon'), (b' ', 'Tab')], help_text='\u0414\u043e\u0441\u0442\u0443\u043f\u043d\u043e \u0442\u0456\u043b\u044c\u043a\u0438 \u0434\u043b\u044f \u0442\u0438\u043f\u0456\u0432: text, input', max_length=10, null=True, verbose_name='Splitter'),
        ),
        migrations.AlterField(
            model_name='configfield',
            name='type',
            field=models.CharField(choices=[(b'input', 'Input'), (b'text', 'Text'), (b'int', 'Integer'), (b'float', 'Float'), (b'bool', 'True / False'), (b'url', 'Url'), (b'email', 'Email'), (b'file', 'File'), (b'image', 'Image'), (b'json', 'JSON')], max_length=50, verbose_name='Type'),
        ),
        migrations.AddField(
            model_name='configfield',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='site_config.ConfigGroup'),
        ),
    ]




from importlib import import_module

from django.apps import apps
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from site_config.models import ConfigField, ConfigGroup
from site_config import config


def _get_config_field_admin_base_class():

    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TranslationAdmin

    return admin.ModelAdmin


class ConfigFieldAdmin(_get_config_field_admin_base_class()):

    CONFIG_FIELDS = ['site', 'group', 'label', 'name', 'type', 'splitter']

    list_display = [
        'label', 'name', 'site', 'type', 'splitter', 'short_value', 'group']

    list_filter = ['site', 'group']

    def _get_value_fields(self, obj=None):

        f_name = obj.value_field_name

        if apps.is_installed('modeltranslation'):

            from modeltranslation.utils import get_translation_fields

            if f_name in self.trans_opts.fields:
                return get_translation_fields(f_name)

        return [f_name]

    def get_form(self, request, obj=None, fields=None, **kwargs):

        if obj is None:
            fields = self.CONFIG_FIELDS
        else:
            self.fieldsets = (
                (
                    _('Value'),
                    {
                        'fields': self._get_value_fields(obj)
                    }
                ),
                (
                    _('Settings'),
                    {
                        'fields': self.CONFIG_FIELDS,
                        'classes': ['collapse'],
                    }
                ),
            )

        if obj is not None and obj.is_html:
            if apps.is_installed('ckeditor_uploader'):
                from ckeditor_uploader.widgets import CKEditorUploadingWidget
                self.formfield_overrides = {
                    models.TextField: {'widget': CKEditorUploadingWidget}
                }

            elif apps.is_installed('ckeditor'):
                from ckeditor.widgets import CKEditorWidget
                self.formfield_overrides = {
                    models.TextField: {'widget': CKEditorWidget}
                }

        return super(ConfigFieldAdmin, self).get_form(
            request, obj=obj, fields=fields, **kwargs)

    def save_model(self, *args, **kwargs):
        super(ConfigFieldAdmin, self).save_model(*args, **kwargs)
        config.reload()


admin.site.register(ConfigField, ConfigFieldAdmin)
admin.site.register(ConfigGroup)




from modeltranslation.translator import translator, TranslationOptions

from site_config.models import ConfigField


class ConfigFieldTranslationOptions(TranslationOptions):
    fields = ('value_input', 'value_text', )


translator.register(ConfigField, ConfigFieldTranslationOptions)




import json

from bs4 import BeautifulSoup

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.contrib.sites.models import Site


FIELD_TYPE_INPUT = 'input'
FIELD_TYPE_TEXT = 'text'
FIELD_TYPE_HTML = 'html'
FIELD_TYPE_JSON = 'json'

FIELD_TYPES = (
    (FIELD_TYPE_INPUT, _("Input")),
    (FIELD_TYPE_TEXT, _("Text")),
    (FIELD_TYPE_HTML, _("HTML")),
    ('int', _("Integer")),
    ('float', _("Float")),
    ('bool', _("True / False")),
    ('url', _("Url")),
    ('email', _("Email")),
    ('file', _("File")),
    ('image', _("Image")),
    (FIELD_TYPE_JSON, _('JSON')),
)


NEWLINE_SPLITTER = 'newline'

SPLIT_CHOICES = (
    (NEWLINE_SPLITTER, _('New line')),
    (',', _('Comma')),
    ('.', _('Dot')),
    (';', _('Semicolon')),
    (' ', _('Tab')),
)


class ConfigGroup(models.Model):

    name = models.CharField(_('Name'), max_length=255, unique=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Config group')
        verbose_name_plural = _('Config groups')


class ConfigField(models.Model):

    SPLIT_TYPES = [FIELD_TYPE_TEXT, FIELD_TYPE_INPUT]

    site = models.ForeignKey(
        Site, verbose_name=_('Site'), default=settings.SITE_ID,
        on_delete=models.CASCADE)

    group = models.ForeignKey(
        ConfigGroup, null=True, blank=True, verbose_name=_('Group'),
        on_delete=models.CASCADE)

    label = models.CharField(_('Label'), max_length=255)

    name = models.CharField(_('Name'), max_length=255)

    type = models.CharField(_('Type'), max_length=50, choices=FIELD_TYPES)

    splitter = models.CharField(
        _('Splitter'), max_length=10, blank=True, null=True,
        choices=SPLIT_CHOICES,
        help_text=_('Available only for types: %s') % ', '.join(SPLIT_TYPES))

    value_input = models.CharField(
        _('Input'), max_length=255, blank=True, null=True)

    value_text = models.TextField(
        _('Text'), max_length=10000, blank=True, null=True)

    value_int = models.IntegerField(_('Integer'), blank=True, null=True)

    value_float = models.IntegerField(_('Float'), blank=True, null=True)

    value_bool = models.NullBooleanField(_('Boolean'), blank=True)

    value_url = models.URLField(_('Url'), blank=True, null=True)

    value_email = models.EmailField(_('Email'), blank=True, null=True)

    value_file = models.FileField(
        _('File'), blank=True, null=True, upload_to='site_config')

    value_image = models.ImageField(
        _('Image'), blank=True, null=True, upload_to='site_config')

    value_json = models.TextField(
        _('Text'), max_length=10000, blank=True, null=True)

    def __str__(self):
        return self.label

    @property
    def value_field_name(self):
        if self.type == FIELD_TYPE_HTML:
            return 'value_%s' % FIELD_TYPE_TEXT
        return 'value_%s' % self.type

    def get_splitter(self):
        splitter = self.splitter

        if splitter == NEWLINE_SPLITTER:
            splitter = '\n'

        return splitter

    @property
    def short_value(self):

        value = self.value

        if isinstance(value, list):
            value = ', '.join(value)

        if self.is_html:
            soup = BeautifulSoup(value, 'html.parser')
            return soup.get_text()[:255]

        if self.type == FIELD_TYPE_TEXT:
            return value[:255]

        return value

    short_value.fget.short_description = _('Short value')

    def _get_value(self):

        value = getattr(self, self.value_field_name)

        if not value:
            return ''

        if self.is_html:
            return mark_safe(value)

        if self.type == FIELD_TYPE_JSON:
            return json.loads(value)

        splitter = self.get_splitter()

        if self.type in self.SPLIT_TYPES and splitter is not None:
            return value.split(splitter)

        return value

    def _set_value(self, value):
        setattr(self, self.value_field_name, value)

    @property
    def is_html(self):
        return self.type == FIELD_TYPE_HTML

    value = property(_get_value, _set_value)

    class Meta:
        unique_together = ['site', 'name']
        ordering = ['label']
        verbose_name = verbose_name_plural = _('settings')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-04-13 05:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=255, verbose_name='Label')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('type', models.CharField(choices=[(b'input', 'Input'), (b'text', 'Text'), (b'int', 'Integer'), (b'float', 'Float'), (b'bool', 'True / False'), (b'url', 'Url'), (b'email', 'Email'), (b'file', 'File'), (b'image', 'Image')], max_length=50, verbose_name='Type')),
                ('splitter', models.CharField(blank=True, choices=[(b'newline', 'New line'), (b',', 'Comma'), (b'.', 'Dot'), (b';', 'Semicolon'), (b' ', 'Tab')], max_length=10, null=True, verbose_name='Splitter')),
                ('value_input', models.CharField(blank=True, max_length=255, null=True, verbose_name='Text')),
                ('value_text', models.TextField(blank=True, max_length=10000, null=True, verbose_name='Text')),
                ('value_int', models.IntegerField(blank=True, null=True, verbose_name='Integer')),
                ('value_float', models.IntegerField(blank=True, null=True, verbose_name='Float')),
                ('value_bool', models.NullBooleanField(verbose_name='Boolean')),
                ('value_url', models.URLField(blank=True, null=True, verbose_name='Url')),
                ('value_email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Email')),
                ('value_file', models.FileField(blank=True, null=True, upload_to=b'site_config', verbose_name='File')),
                ('value_image', models.ImageField(blank=True, null=True, upload_to=b'site_config', verbose_name='Image')),
            ],
            options={
                'ordering': ['type'],
                'verbose_name': 'settings',
                'verbose_name_plural': 'settings',
            },
        ),
    ]
